(* *********************************************************** *)
(*                                                             *)
(*****************        FeynArts 1.0        ******************)
(*                                                             *)
(*   Copyright by Sepp Kueblbeck and Hagen Eck 1991            *)
(*   algorithms by Ansgar Denner                               *)
(*                                                             *)
(* *********************************************************** *)

(* *********************************************************** *)
(*							       *)
(*  Release notes :					       *)
(*                                                             *)
(*  FeynArts is a public domain program package.               *)
(*  You may make and distribute copies of FeynArts             *)
(*  provided the code of the distributed copies is             *)
(*  identical to the original in all files of FeynArts.        *)
(*  Please make sure that the users you give FeynArts to       *)
(*  have access to a FeynArts manual.                          *)
(*                                                             *)
(*  We can not guarantee that the functions of FeynArts        *)
(*  are completely free of errors. If you find out that        *)
(*  FeynArts does not work as described in the manual          *)
(*  please send a bug report to:                               *)
(*                                                             *)
(*           Hagen Eck, Sepp Kueblbeck                         *)
(*           Physikalisches Institut			       				*)
(*           Universitaet Wuerzburg			       					*)
(*           Am Hubland					       							*)
(*           D-8700 Wuerzburg				      					 	*)
(*           E-mail: eck@vax.rz.uni-wuerzburg.dbp.de           *)
(*               or: kue@vax.rz.uni-wuerzburg.dbp.de           *)
(*							       												*)
(* We also welcome any kind of suggestions for improving       *)
(* FeynArts.         					       							*)
(*                                                             *)
(* *********************************************************** *)
(* technical note:					       								*)
(* make sure that the directory of this file also contains     *)
(* the files General.m CreateTopologies.m InsertFields.m       *)
(* CreateFeynAmp.m and Paint.m as well as the subdirectories   *)
(* Utilities, GraphInfo and Models.                            *)
(* Before loading FeynArts.m into Mathematica the $Path has    *)
(* to contain this directory.                   	       		*)
(* Have fun!						      								 	*)	
(* *********************************************************** *)


Begin["FeynArts`"]

(* *********************************************************** *)
(* TextFonts and PrinterFonts for the Feynman graph output     *)

$ScreenTextFont = {"Courier",10};             
	         (* for Latin letters on screen                		*)
$SmallScreenTextFont = {"Courier",8};   
                 (* for Latin letters on screen for subscript  *)
$ScreenSymbolFont = None; 
	         (* for Greek letters on screen                		*)	
$SmallScreenSymbolFont = None; 
	         (* for Greek letters on screen for subscript  		*)	
$PrinterTextFont = {"Helvetica",10};
	         (* for Latin letters on printer               		*)	
$SmallPrinterTextFont = {"Helvetica",8};
	         (* for Latin letters on printer for subscript 		*)	
$PrinterSymbolFont = {"Symbol",10};
	         (* for Greek letters on printer               		*)	
$SmallPrinterSymbolFont = {"Symbol",8};
	         (* for Greek letters on printer for subscript 		*)	

$DefaultFont=$PrinterTextFont;
                 (* Font to be used as LabelFont below graphs  *)

$ScreenTextFont = {"Helvetica",7};   
$SmallScreenTextFont = {"Helvetica",5};   
$ScreenSymbolFont = {"Symbol",7};
$SmallScreenSymbolFont = {"Symbol",5};
$DefaultFont={"Helvetica",7};


(* Translation rules if no SymbolFont is available on screen 
   or on printer 					       *)	
 
NoGreek["c"]="x";            (* converts a Greek chi   to "x"  *)
NoGreek["g"]="A";            (* converts a Greek gamma to "A"  *)
NoGreek["j"]="fi";           (* converts a Greek fi    to "fi" *)
NoGreek["m"]="m";            (* converts a Greek mu    to "m"  *)
NoGreek["n"]="n";            (* converts a Greek nu    to "n"  *)
NoGreek["t"]="l";            (* converts a Greek tau   to "l"  *)

(* *********************************************************** *)
(* Field and Model variables 				       *)	

Models      = {  SM , QED , QCD , QE , QC };
                   (* the models which FeynArts knows of       *)
SingleModels= {  QE , QC };
                   (* models which can not be combined 
		      with any of the other Models             *)
Fieldtypes  = {  F , G , S , V , U };
                   (* field variables FeynArts knows of        *)
		   (* fermions, gluons, scalars fields,        *)
		   (* vector fields, ghost fields              *)

(* *********************************************************** *)
(* set the photon mass MLA = 0                                 *)
(* (ommit this line for infrared divergent integrals)          *)

Global`MLA = 0;

(* *********************************************************** *)
(* File used for writing screen messages                       *)

FeynArtsNullDevice = "FeynArts.log";
Open[FeynArtsNullDevice]

(* *********************************************************** *)
End[]

Print[" "];
Print["FeynArts 1.0 for Mathematica 2.0 (October 1991)"];
Print["by Hagen Eck and Sepp Kueblbeck "];
Print["[additional packages being loaded]"];

(* *********************************************************** *)
(* Loading the other packages, for DOS which has only 8        *)
(* letters for a file name and three for its extension         *)
(* the paths are changed here:                                 *)

If[ 
  $OperatingSystem==="MS-DOS",
  Unprotect[ ContextToFilename ];
  ContextToFilename[ "Utilities`Compare`" ] = 
		   "UTILITIE\COMPARE.M";
  ContextToFilename[ "Utilities`AddMomenta`" ] =
		   "UTILITIE\ADDMOMEN.M";
  ContextToFilename[ "Utilities`DrawLine`" ] = 
		   "UTILITIE\DRAWLINE.M";
  ContextToFilename[ "Utilities`FeynGrassman`" ] = 
		   "UTILITIE\FEYNGRASS.M";
  ContextToFilename[ "Utilities`FeynMultiplication`" ] = 
		   "UTILITIE\FEYNMULT.M";
  ContextToFilename[ "Utilities`OutputRules`" ] = 
		   "UTILITIE\OUTPUTRU.M"; 
  ContextToFilename[ "Utilities`GraphPoints`" ] = 
		   "UTILITIE\GRAPHPOI.M";
  ContextToFilename[ "Utilities`VertexHandling`" ] = 
		   "UTILITIE\VERTEXHA.M";
  ContextToFilename[ "Utilities`FeynInit`" ] = 
		   "UTILITIE\FEYNINIT.M";
  ContextToFilename[ "GraphInfo`GraphInfo`" ] = 
		   "GRAPHINF\GRAPHINF.M";
  <<GENERAL.M;
  <<CREATETO.M;
  <<INSERTFI.M;
  <<CREATEFE.M;
  <<PAINT.M     ,
  <<General.m;
  <<CreateTopologies.m;
  <<InsertFields.m;
  <<CreateFeynAmp.m;
  <<Paint.m 
  ];

(* *********************************************************** *)

Null
